#include "dashboard.h"

/* constructor of the Dasboard class */
Dashboard::Dashboard( QWidget * parent ) : QMainWindow( parent )
{
	ui.setupUi( this );

	InitViewer( QStringLiteral( u"../../test/test.uia" ) ); // load the presentation from the uia file
	InitControls();
}

/* this slot is invoked when meshes are created */
void Dashboard::meshesCreated( const QStringList & meshNames, const QString & error )
{
	qDebug() << meshNames;
	qDebug() << error;
}

/* this slot is invoked when elements are created */
void Dashboard::elementsCreated( const QStringList & elementPaths, const QString & error )
{
	qDebug() << elementPaths;
	qDebug() << error;
}

/* initialization procedures take place here */
int Dashboard::InitViewer( const QString & source_file_name )
{
	const QUrl source_url = QUrl::fromLocalFile( source_file_name );
	qDebug() << "Source file" << source_url;

	QSurfaceFormat::setDefaultFormat( Q3DS::surfaceFormat() );

	if ( !context_.create() )
	{
		qFatal( "Failed to create OpenGL context" );
	}

	window_.setSurfaceType( QSurface::OpenGLSurface );
	window_.setFormat( context_.format() );
	window_.create();

	QObject::connect( &viewer_, &Q3DSSurfaceViewer::presentationLoaded, &viewer_, [&]
		{
			qDebug() << "Presentation loaded";
		} );

	// get all data inputs we need to control the items in the presentation
	QObject::connect( &viewer_, &Q3DSSurfaceViewer::presentationReady, &viewer_, [&]
		{
			qDebug() << "Presentation ready";

			const auto dataInputs = viewer_.presentation()->dataInputs();

			for ( const auto & it : dataInputs )
			{
				Q3DSDataInput * dataInput = it;
				qDebug() << "Processing data input " << dataInput->name();

				// if we recognize the data input
				if ( dataInput->name() == QString( "SphereSize" ) )
				{
					// we store it for later use
					diSphereSize_ = dataInput;
				}
			}
		} );

	viewer_.presentation()->setSource( source_url );
	viewer_.setUpdateInterval( 0 ); // enable automatic updates

	viewer_.create( &window_, &context_ );

	window_.setTitle( QStringLiteral( "Dashboard" ) );
	window_.resize( 800, 480 );
	window_.show();

	return 0;
}

/* all the widgets controlling the dashboard come here */
int Dashboard::InitControls()
{
	ui.centralWidget->setLayout( new QBoxLayout( QBoxLayout::Direction::TopToBottom, ui.centralWidget ) );
	
	QLabel * lblSize = new QLabel( ui.centralWidget );
	lblSize->setText( QStringLiteral( "Sphere size" ) );	
	ui.centralWidget->layout()->addWidget( lblSize );

	QSpinBox * spbSize = new QSpinBox( ui.centralWidget );
	spbSize->setMinimum( 1 );
	spbSize->setMaximum( 10 );
	spbSize->setMaximumWidth( 50 );
	ui.centralWidget->layout()->addWidget( spbSize );

	QObject::connect( spbSize, QOverload<int>::of( &QSpinBox::valueChanged ),
		[=]( int i )
		{
			// if we know the data input
			if ( spbSize )
			{
				// we can set the size property of the sphere
				diSphereSize_->setValue( QVector3D( float( i ), float( i ), float( i ) ) * 0.1f );
				qDebug() << diSphereSize_->value();
			}
		} );	

	return 0;
}
